import {Inject, Injectable, LOCALE_ID, Optional} from '@angular/core';

import {MOMENT, TIMEZONE_OVERRIDE} from '../../common/interfaces/Tokens';
import {DayWithMetadata} from '../interfaces/DayWithMetadata';
import {DateRangeValidationParams} from '../interfaces/DateRangeValidationParams';
import {DateAdapter} from '../services/date-picker.service';

/* Note: Typings of 'any' in this file refer to Moment.
         The Moment type does not come without the full weight of Moment.js, so it was intentionally left out. */

@Injectable()
// Adapter for Date-Picker to accept String, Native Date or Moment objects
export class MomentDateAdapter implements DateAdapter<any> {

    private moment: any;
    private timezone: string;
    private formatterLanguage: string;

    constructor(
        @Inject(MOMENT) private _moment: any,
        @Optional() @Inject(LOCALE_ID) private _locale?: string,
        @Optional() @Inject(TIMEZONE_OVERRIDE) private _timezone?: string) {

        this.moment = _moment;
        this.timezone = _timezone;

        if (this.moment) {
            this.moment.locale(this._locale);

            if (this.timezone) {
                this.moment.tz.setDefault(this.timezone);
            } else {
                console.warn('Timezone is not set, is defaulting to system timezone. TIMEZONE_OVERRIDE may be set at the Module level.');
            }
        } else {
            console.error('Package "moment" has not been installed. Please either install "moment" or use the NativeDateAdapter.');
        }
    }

    public dayOfFirst(year: number, month: number): number {
        const adjusted = this.handleMonthAdjustment(year, month);

        return this.moment([adjusted.year, adjusted.month, 1]).day();
    }

    public daysInMonth(year: number, month: number): number {
        const adjusted = this.handleMonthAdjustment(year, month);

        return this.moment([adjusted.year, adjusted.month]).daysInMonth();
    }

    public isDisabled(evalDate: Date, dateRangeParams: DateRangeValidationParams): boolean {
        if (!dateRangeParams || (!dateRangeParams.minDate && !dateRangeParams.maxDate)) { return false; }

        const dayIsBeforeMinDate = this.moment(evalDate).isBefore(this.moment(dateRangeParams.minDate), 'day');
        const dayIsAfterMaxDate = this.moment(evalDate).isAfter(this.moment(dateRangeParams.maxDate), 'day');

        return dayIsBeforeMinDate || dayIsAfterMaxDate;
    }

    public getMonthInfo(year: number, month: number): Date {
        const adjusted = this.handleMonthAdjustment(year, month);

        return new Date(this.moment([adjusted.year, adjusted.month, 15]));
    }

    public month(date: Date): number {
        return this.moment(date).month();
    }

    public newDateFromArray(year?: number, month: number = 0, day: number = 1): any {
        return new Date(this.moment([year, month, day]));
    }

    public newDateFromValue(value?: any): any {
        if (!value) {
            return new Date();
        } else if (value instanceof Date) {
            return new Date(this.moment(value));
        } else {
            return new Date(value);
        }
    }

    public prepHeaderDate(date: Date): string {
        return this.moment(date).locale(this.formatterLanguage || this._locale).format('ddd, MMM D');
    }

    public prepHeaderYear(date: Date): string {
        return this.moment(date).locale(this.formatterLanguage || this._locale).format('YYYY');
    }

    public sameDay(_calendarDay: DayWithMetadata, _comparator: Date): boolean {
        const calendarDay = this.moment([_calendarDay.year, _calendarDay.month, _calendarDay.day]);
        const comparator = this.moment(_comparator);

        return calendarDay.isSame(comparator, 'day');
    }

    public year(date: Date): number {
        return this.moment(date).year();
    }

    public weekdays(): string[] {
        return this.moment.weekdays();
    }

    public months(): string[] {
        return this.moment.monthsShort();
    }

    private handleMonthAdjustment(year: number, month: number): {year: number, month: number} {
        if (month > 11) {
            month = month - 12;
            year++;
        } else if (month < 0) {
            month = month + 12;
            year--;
        }

        return {year, month};
    }

    public prepDateStringOutput(input: any): string {
        return (input) ? this.moment(this.processDateInput(input)).format('l') : '';
    }

    public processDateInput(input: any): Date {
        if (input && this.moment(input, 'L', this._locale).isValid()) {
            const timezone = this.timezone || this.moment.tz.guess();
            return this.moment(input, 'L', this._locale).tz(timezone).startOf('day').toDate();
        }

        return new Date('false');
    }
}

export function getMomentDateAdapter(MOMENT_: any, locale?: string, timezone?: string): MomentDateAdapter {
    return new MomentDateAdapter(MOMENT_, locale, timezone);
}

