import {Inject, Injectable, LOCALE_ID, Optional} from '@angular/core';

import {DayWithMetadata} from '../interfaces/DayWithMetadata';
import {DateRangeValidationParams} from '../interfaces/DateRangeValidationParams';
import {DateAdapter, getMonths, localize, parseInput, getWeekdays} from '../services/date-picker.service';

@Injectable()
// Adapter for UTC dates
export class UTCDateAdapter implements DateAdapter<Date> {
    private readonly languageCode: string;

    constructor(@Optional() @Inject(LOCALE_ID) private _locale?: string) {
        this.languageCode = (_locale) ? _locale.split('-')[0] : 'en';
    }

    public dayOfFirst(year: number, month: number): number {
        return new Date(year, month, 1).getDay();
    }

    public daysInMonth(year: number, month: number): number {
        return new Date(year, month + 1, 0).getDate();
    }

    public isDisabled(evalDate: Date, dateRangeParams: DateRangeValidationParams): boolean {
        if (!dateRangeParams) { return false; }

        const dayIsBeforeMinDate = (dateRangeParams.minDate) ? evalDate < dateRangeParams.minDate : false;
        const dayIsAfterMaxDate = (dateRangeParams.maxDate) ? evalDate > dateRangeParams.maxDate : false;

        return dayIsBeforeMinDate || dayIsAfterMaxDate;
    }

    public getMonthInfo(year: number, month: number): Date {
        return new Date(year, month, 15);
    }

    public month(date: Date): number {
        return date.getUTCMonth();
    }

    public newDateFromArray(year?: number, month?: number, day?: number): Date {
        return new Date(Date.UTC(year, month, day));
    }

    public newDateFromValue(value?: any): Date {
        return (value) ? this.processDateInput(value) : this.getZerodDate(new Date());
    }

    public prepHeaderDate(date: Date): string {
        return localize(this.languageCode, this.addTimezoneOffset(date), 'EEE, MMM d');
    }

    public prepHeaderYear(date: Date): string {
        return localize(this.languageCode, this.addTimezoneOffset(date), 'yyyy');
    }

    public sameDay(_calendarDay: DayWithMetadata, _comparator: Date): boolean {
        return _calendarDay.day === _comparator.getUTCDate()
            && _calendarDay.month === _comparator.getUTCMonth()
            && _calendarDay.year === _comparator.getUTCFullYear();
    }

    public year(date: Date): number {
        return date.getUTCFullYear();
    }

    public weekdays(): string[] {
        return getWeekdays(this.languageCode);
    }

    public months(): string[] {
        return getMonths(this.languageCode);
    }

    public prepDateStringOutput(input: Date): string {
        return (input) ? new Date(this.processDateInput(input)).toLocaleDateString(this._locale, {timeZone: 'UTC'}) : '';
    }

    public processDateInput(input: Date | string): Date {
        if (!input) { return null; }

        if (typeof input === 'string') {
            const dateSegments = parseInput(input, this._locale);

            if (Array.isArray(dateSegments.splitDate) && dateSegments.splitDate.length === 3 && dateSegments.year) {
                return new Date(Date.UTC(
                    parseInt(dateSegments.year, 10),
                    parseInt(dateSegments.month, 10) - 1,
                    parseInt(dateSegments.day, 10))
                );
            } else {
                // This is a fallback for malformed date string, outcome unknown
                return new Date(input);
            }
        } else {
            return this.getZerodDate(input);
        }
    }

    private addTimezoneOffset(date: Date): Date {
        const newDate = new Date(date);
        const timeOffsetInMS = date.getTimezoneOffset() * 60000;

        newDate.setTime(date.getTime() + timeOffsetInMS);
        return newDate;
    }

    private getZerodDate(date: any): Date {
        const safeDate = (date._isAMomentObject) ? new Date(date) : date;

        const day = safeDate.getUTCDate();
        const month = safeDate.getUTCMonth();
        const year = safeDate.getUTCFullYear();

        return new Date(Date.UTC(year, month, day));
    }
}
