import {
    ChangeDetectorRef,
    Directive,
    Input,
    Optional,
    Self,
    OnDestroy,
    ElementRef,
    EventEmitter,
    HostListener,
    HostBinding,
    Inject,
    Renderer2,
    AfterViewInit,
    LOCALE_ID
} from '@angular/core';
import {ControlValueAccessor, FormGroupDirective, NgControl, NgForm} from '@angular/forms';
import {DateAdapter} from '../services/date-picker.service';
@Directive({
    selector: 'anj-input[anjDatePicker]',
    exportAs: 'anjDatepickerInput'
})


export class DatePickerDirective implements OnDestroy, ControlValueAccessor, AfterViewInit {
    // Custom input to return inner value, but display masked value
    @Input()
    get value(): Date { return this.innerDateValue; }
    set value(dateValue: Date) {
        if (dateValue) {
            this.innerDateValue = this.adapterStrategy.processDateInput(dateValue);
            this.outerValue = this.adapterStrategy.prepDateStringOutput(dateValue);
            this.changeRef.markForCheck();
        }
    }

    // AdapterStrategy will be set by the DateAdapter injected provider. This is for an optional override.
    @Input()
    set adapter(adapter: any) {
        this.adapterStrategy = adapter;
    }
    @HostBinding('value') public outerValue: string;
    @HostBinding('attr.invalid') public invalid: boolean;
    @HostBinding('attr.language') public languageCode: string;
    public adapterStrategy: DateAdapter<Date>;
    public readonly valueChange: EventEmitter<Date | null>;

    private innerDateValue: Date;
    private listenerFn: () => void;
    private readonly parent: NgForm | FormGroupDirective | null;

    constructor(private renderer: Renderer2,
        private _element: ElementRef,
                private changeRef: ChangeDetectorRef,
                public dateAdapter: DateAdapter<Date>,
                @Optional() parentForm: NgForm,
                @Optional() parentFormGroup: FormGroupDirective,
                @Optional() @Self() private ngControl?: NgControl,
                @Optional() @Inject(LOCALE_ID) _locale?: string) {
        this.languageCode = (_locale) ? _locale.split('-')[0] : 'en';
        this.valueChange = new EventEmitter<Date | null>();

        this.adapterStrategy = dateAdapter;

        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }

        this.parent = parentForm ? parentForm : parentFormGroup ? parentFormGroup : null;
    }

    public ngAfterViewInit() {
        if (this.ngControl) {
            this.value = this.ngControl.value;
            this.changeRef.detectChanges();
        }
    }

    public ngOnDestroy(): void {
        if (this.listenerFn) {
            this.listenerFn();
        }
    }

    // // Start: ControlValueAccessor requirements
    public writeValue(newValue: Date): void {
        this.value = newValue;
    }

    public onChange: Function = () => {
    }

    public onTouch: Function = () => {
    }

    public registerOnChange(fn: any): void {
        this.onChange = fn;
    }

    public registerOnTouched(fn: any): void {
        this.onTouch = fn;
    }
    // // End: ControlValueAccessor requirements

    @HostListener('anjBlur', ['$event'])
    public onInput(event: any) {
        if (event.target.tagName === 'ANJ-INPUT') {
            this.onTouch();
            this.updateInnerValue(event.target.value);
            this.valueChange.emit(this.innerDateValue);
        }
    }

    @HostListener('anjCalendarSelection', ['$event'])
    public onCalendarSelect(event: any) {
        this.onTouch();
        this.updateInnerValue(event.detail);
        this.valueChange.emit(this.innerDateValue);
    }

    /* Intercept and override native input validity, if ngControl has a validity issue */
    @HostListener('anjInvalid', ['$event'])
    public onAnjInvalid(event: any) {
        this.isInvalid();

        if (event.detail !== this.invalid && !event.detail) {
            this._element.nativeElement
                .dispatchEvent(new CustomEvent('anjInvalid', {
                    detail: this.invalid,
                    bubbles: true
                }));

            this.changeRef.markForCheck();
        }
    }

    private isInvalid(): boolean {
        if (!this.ngControl) {
            return;
        }

        const isSubmitted = (this.parent && this.parent.submitted);
        this.invalid = this.ngControl.invalid && (this.ngControl.touched || isSubmitted);
    }

    // convert the masked value to storage value
    private updateInnerValue(inputDate: string | Date): void {
        this.innerDateValue = this.adapterStrategy.processDateInput(inputDate);
        this.propogateInnerValueChange();
    }

    private propogateInnerValueChange(): void {
        if (this.ngControl) {
            this.ngControl.control.setValue(this.innerDateValue);
        }
        this.valueChange.emit(this.innerDateValue);
    }
}
