import {Inject, LOCALE_ID, Optional} from '@angular/core';
import {DatePipe} from '@angular/common';

import {DateSegments} from '../interfaces/DateSegments';
import {DayWithMetadata} from '../interfaces/DayWithMetadata';
import {DateRangeValidationParams} from '../interfaces/DateRangeValidationParams';

export const STANDARD_LOCALES: {[key: string]: string} = {
    'en-US': 'M/d/yyyy',
    'sw-KE': 'M/d/yyyy',
    'en-ZW': 'M/d/yyyy',
    'en-PH': 'M/d/yyyy',
    'rw-RW': 'M/d/yyyy',
    'moh-CA': 'M/d/yyyy',
    'fil-PH': 'M/d/yyyy',
    'ne-NP': 'M/d/yyyy',
    'es-US': 'M/d/yyyy',
    'sq-AL': 'yyyy/M/d',
    'bg-BG': 'yyyy/M/d',
    'sr-BA': 'yyyy/M/d',
    'fr-CA': 'yyyy/M/d',
    'zh-CN': 'yyyy/M/d',
    'zh-HK': 'yyyy/M/d',
    'hu-HU': 'yyyy/M/d',
    'ja-JP': 'yyyy/M/d',
    'ko-KR': 'yyyy/M/d',
    'lt-LT': 'yyyy/M/d',
    'sv-SE': 'yyyy/M/d',
    'zh-TW': 'yyyy/M/d',
    'en-ZA': 'yyyy/M/d'
};

export abstract class DateAdapter<T> {
    // For calendar widget
    public abstract dayOfFirst(year: number, month: number): number;
    public abstract daysInMonth(year: number, month: number): number;
    public abstract getMonthInfo(year: number, month: number): T;
    public abstract isDisabled(evalDate: Date, dateRangeParams: DateRangeValidationParams): boolean;
    public abstract month(date: Date): number;
    public abstract newDateFromArray(year?: number, month?: number, day?: number): T;
    public abstract newDateFromValue(value?: any): T;
    public abstract prepHeaderDate(date: Date, weekDays?: string[], months?: string[]): string;
    public abstract prepHeaderYear(date: Date, weekDays?: string[], months?: string[]): string;
    public abstract sameDay(_calendarDay: DayWithMetadata, _comparator: Date): boolean;
    public abstract year(date: Date): number;
    public abstract weekdays(): string[];
    public abstract months(): string[];

    // For angular directive
    public abstract prepDateStringOutput(input: T): string;
    public abstract processDateInput(input: T | string): Date;
}

// Default adapter, will accept String or Native Date
export class NativeDateAdapter implements DateAdapter<Date> {
    private languageCode: string;

    constructor(@Optional() @Inject(LOCALE_ID) private _locale?: string) {
        this.languageCode = (_locale) ? _locale.split('-')[0] : 'en';
    }

    dayOfFirst(year: number, month: number): number {
        return new Date(year, month, 1).getDay();
    }

    daysInMonth(year: number, month: number): number {
        return new Date(year, month + 1, 0).getDate();
    }

    isDisabled(evalDate: Date, dateRangeParams: DateRangeValidationParams): boolean {
        if (!dateRangeParams) { return false; }

        const dayIsBeforeMinDate = (dateRangeParams.minDate) ? evalDate < dateRangeParams.minDate : false;
        const dayIsAfterMaxDate = (dateRangeParams.maxDate) ? evalDate > dateRangeParams.maxDate : false;

        return dayIsBeforeMinDate || dayIsAfterMaxDate;
    }

    getMonthInfo(year: number, month: number): Date {
        return new Date(year, month, 15);
    }

    month(date: Date): number {
        return date.getMonth();
    }

    newDateFromArray(year?: number, month?: number, day?: number): Date {
        return new Date(year, month, day);
    }

    newDateFromValue(value?: any) {
        return (value) ? new Date(value) : new Date();
    }

    prepHeaderDate(date: Date): string {

        return localize(this.languageCode, date, 'EEE, MMM d');
    }

    prepHeaderYear(date: Date): string {

        return localize(this.languageCode, date, 'yyyy');
    }

    sameDay(_calendarDay: DayWithMetadata, _comparator: Date): boolean {
        return _calendarDay.day === _comparator.getDate()
            && _calendarDay.month === _comparator.getMonth()
            && _calendarDay.year === _comparator.getFullYear();
    }

    year(date: Date): number {
        return date.getFullYear();
    }

    weekdays(): string[] {
        return getWeekdays(this.languageCode);
    }

    months(): string[] {
        return getMonths(this.languageCode);
    }

    prepDateStringOutput(input: Date): string {
        return (input) ? new Date(this.processDateInput(input)).toLocaleDateString(this._locale) : '';
    }

    processDateInput(input: Date | string): Date {
        if (!input) { return null; }

        if (typeof input === 'string') {
            const dateSegments = parseInput(input, this._locale);

            if (Array.isArray(dateSegments.splitDate) && dateSegments.splitDate.length === 3 && dateSegments.year) {
                return new Date(parseInt(dateSegments.year, 10), parseInt(dateSegments.month, 10) - 1, parseInt(dateSegments.day, 10));
            } else {
                return new Date(input);
            }
        } else {
            return input;
        }
    }
}

export function getWeekdays(lang: string): string[] {
    const weekdays = [];
    for (let i = 0; i < 7; i++) {
        const date = new Date();
        const testDate = date.setDate(date.getDate() - date.getDay() + i);
        weekdays.push(localize(lang, new Date(testDate), 'EEE'));
    }
    return weekdays;
}

export function getMonths(lang: string): string[] {
    const months = [];
    for (let i = 0; i < 12; i++) {
        const testDate = new Date().setMonth(i);
        months.push(localize(lang, new Date(testDate), 'MMM'));
    }
    return months;
}

export function localize(_languageCode: string = 'en', date: Date, format: string): string {
    const pipe = new DatePipe(_languageCode);
    return pipe.transform(date, format);
}


/* Currently only support shortDate strings (with non-alphanumeric separators) of:
    {
        Month-Day-Year: if included in STANDARD_LOCALES,
        Year-Month-Day: if included in STANDARD_LOCALES,
        Day-Month-Year: for all locales outside of STANDARD_LOCALES
    }
 */
export function parseInput(input: string, locale: string): DateSegments {
    let format = STANDARD_LOCALES[locale];
    // Covers case of partial locale code provided
    if (!format) {
        const localeKey = Object.keys(STANDARD_LOCALES).find(key => key.includes(locale));
        format = STANDARD_LOCALES[localeKey];
    }

    // Split string on any non-alphanumeric characters, not including spaces
    const regexSplit = input.split(/[^\w\s]+/g).filter(Boolean);

    // Parse digit-only string, or use the regexSplit value
    const splitDate = (regexSplit.length <= 1 && input.length === 8) ? parseDigitOnlyDate(input, format) : regexSplit;

    const dateSegments = {
        splitDate,
        year: '',
        month: '',
        day: ''
    };

    const splitDateCp = splitDate.slice(0);

    // Set year
    dateSegments.year = setYearSegment(splitDateCp);

    // Set Day and Month
    Object.assign(dateSegments, setMonthAndDaySegments(splitDateCp, format));

    return dateSegments;
}

function setYearSegment(splitDateArray: string[]): string {
    // Find year
    const year = splitDateArray.find(segment => segment.length === 4);

    // Remove year from working array
    const yearIndex = splitDateArray.indexOf(year);
    splitDateArray.splice(yearIndex, 1);

    return year;
}

function setMonthAndDaySegments(splitDateArray: string[], format: string): {day: string; month: string} {
    const segments = {
        day: '',
        month: ''
    };

    // See if we can determine which segment is the days segment
    const assumedDay = splitDateArray.find(segment => parseInt(segment, 10) > 12);

    if (assumedDay) {
        segments.day = assumedDay;
        const dayIndex = splitDateArray.indexOf(assumedDay);
        splitDateArray.splice(dayIndex, 1);
        segments.month = splitDateArray[0];
        // Otherwise, make a reasonable guess
    } else {
        switch (format) {
            case 'M/d/yyyy':
            case 'yyyy/M/d':
                segments.month = splitDateArray[0];
                segments.day = splitDateArray[1];
                break;
            // Assume 'dd/MM/yyyy' for anything else
            default:
                segments.month = splitDateArray[1];
                segments.day = splitDateArray[0];
                break;
        }
    }

    return segments;
}

function parseDigitOnlyDate(input: string, format: string): string[] {
    let splitDate: string[];

    switch (format) {
        case 'yyyy/M/d':
            splitDate = [
                input.slice(0, 4),
                input.slice(4, 6),
                input.slice(6)
            ];
            break;
        // Assume 'dd/MM/yyyy' or 'M/d/yyyy' for anything else
        default:
            splitDate = [
                input.slice(0, 2),
                input.slice(2, 4),
                input.slice(4)
            ];
            break;
    }

    return splitDate;
}
