import {AbstractControl} from '@angular/forms';

import {AnjunaValidators} from '@anjuna/angular-core';

const mockControl: any = {
    value: null
};

describe('validator: required-check', () => {
    const validator: (c: AbstractControl) => any = AnjunaValidators.requiredCheck;

    it('should return null when the control has a value of true', () => {
        mockControl.value = true;
        const error = validator(mockControl);

        expect(error).toEqual(null);
    });

    it('should return an error object when the control has a value of false', () => {
        mockControl.value = false;
        const error = validator(mockControl);

        expect(error).toEqual({
            required: {
                valid: false
            }
        });
    });

    it('should return an error object when the control has a value of null', () => {
        mockControl.value = null;
        const error = validator(mockControl);

        expect(error).toEqual({
            required: {
                valid: false
            }
        });
    });
});

describe('validator: date', () => {
    const validator: (c: AbstractControl) => any = AnjunaValidators.date;

    it('should return nothing if there is no control', () => {
        const empty: any = {};
        const error = validator(empty);
        expect(error).toBeFalsy();
    });

    it('should return null when the control has a valid date', () => {
        mockControl.value = new Date();
        const error = validator(mockControl);

        expect(error).toEqual(null);
    });

    it('should return an error when it is not a valid date', () => {
        mockControl.value = new Date('pete');
        const error = validator(mockControl);

        expect(error).toEqual({
            date: {
                valid: false
            }
        });
    });

    it('should return an error when it is a string that cannot be a date', () => {
        mockControl.value = 'pete';
        const error = validator(mockControl);

        expect(error).toEqual({
            date: {
                valid: false
            }
        });
    });

    it('should handle moment objects', () => {
        mockControl.value = {
            _isValid: false
        };

        const error = validator(mockControl);

        expect(error).toEqual({
            date: {
                valid: false
            }
        });
    });
});

describe('validator: maxDate', () => {
    const now = new Date();
    const validator: (c: AbstractControl) => any = AnjunaValidators.maxDate(now);

    it('should return null when the control has a date = maxDate', () => {
        mockControl.value = now;
        const error = validator(mockControl);

        expect(error).toEqual(null);
    });

    it('should return null when the control has a date < maxDate', () => {
        mockControl.value = new Date(new Date().setDate(new Date().getDate() - 5));
        const error = validator(mockControl);

        expect(error).toEqual(null);
    });

    it('should return error when the control has a date > maxDate', () => {
        mockControl.value = new Date(new Date().setDate(new Date().getDate() + 5));
        const error = validator(mockControl);

        expect(error).toEqual({
            max: {
                valid: false
            }
        });
    });

    it('should return undefined when there is no value', () => {
        mockControl.value = undefined;
        const error = validator(mockControl);

        expect(error).toEqual(undefined);
    });
});

describe('validator: minDate', () => {
    const now = new Date();
    const validator: (c: AbstractControl) => any = AnjunaValidators.minDate(now);

    it('should return null when the control has a date = minDate', () => {
        mockControl.value = now;
        const error = validator(mockControl);

        expect(error).toEqual(null);
    });

    it('should return null when the control has a date > minDate', () => {
        mockControl.value = new Date(new Date().setDate(new Date().getDate() + 5));
        const error = validator(mockControl);

        expect(error).toEqual(null);
    });

    it('should return error when the control has a date < minDate', () => {
        mockControl.value = new Date(new Date().setDate(new Date().getDate() - 5));
        const error = validator(mockControl);

        expect(error).toEqual({
            min: {
                valid: false
            }
        });
    });

    it('should return undefined when there is no value', () => {
        mockControl.value = undefined;
        const error = validator(mockControl);

        expect(error).toEqual(undefined);
    });
});


describe('validator: unique', () => {
    const validator: (c: AbstractControl) => any = AnjunaValidators.unique;

    it('should return undefined when the control a non-Array value', () => {
        mockControl.value = 'string';
        const error = validator(mockControl);

        expect(error).toEqual(undefined);
    });

    it('should return null when the control has an empty list', () => {
        mockControl.value = [];
        const error = validator(mockControl);

        expect(error).toEqual(null);
    });

    it('should return null when the control has a unique list', () => {
        mockControl.value = [1, 2];
        const error = validator(mockControl);

        expect(error).toEqual(null);
    });

    it('should return error when the control has duplicate value in the list', () => {
        mockControl.value = [1, 1];
        const error = validator(mockControl);

        expect(error).toEqual({
            unique: {
                valid: false
            }
        });
    });
});

