import {DateAdapter, localize, NativeDateAdapter, UTCDateAdapter} from '@anjuna/angular-core';

const moment = require('moment-timezone');

import {registerLocaleData} from '@angular/common';
import localeJA from '@angular/common/locales/ja';
registerLocaleData(localeJA);


describe('service: NativeDateAdapter locale: en-US', () => {
    let adapter: DateAdapter<any>;

    beforeEach(() => {
        adapter = new NativeDateAdapter( 'en-US');
    });

    it('should default a language if no locale is provided', () => {
        const test = localize(undefined, new Date(2017, 1, 1), 'yyyy');
        expect(test).toEqual('2017');
    });

    it('should process Date input of type Date properly', () => {
        const testDate = new Date('12/02/2001');
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(testDate);
    });

    it('should process Date input of null', () => {
        const test = adapter.processDateInput(null);
        expect(test).toEqual(null);
    });

    it('should process Date input of shortDate string properly for locale, slash separation', () => {
        const testDate = '12/02/2001';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 11, 2));
    });

    it('should process Date input of shortDate string properly for locale, dash separation', () => {
        const testDate = '12-02-2001';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 11, 2));
    });

    it('should process Date input of shortDate string properly for locale, period separation', () => {
        const testDate = '12.02.2001';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 11, 2));
    });

    it('should process Date input of digit-only string properly for locale', () => {
        const testDate = '12022001';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 11, 2));
    });

    it('should process Date input of type string properly for locale', () => {
        const testDate = 'Dec 2 2001';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 11, 2));
    });

    it('should process ambigious short-string dates predictably', () => {
        let testDate = '1/5/2018';
        let test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 5));

        testDate = '5/1/2018';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 4, 1));

        testDate = '1/20/2018';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 20));

        testDate = '20/1/2018';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 20));

        testDate = '2018/1/5';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 5));

        testDate = '2018/5/1';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 4, 1));

        testDate = '2018/1/20';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 20));

        testDate = '2018/20/1';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 20));
    });

    it('should identify invalid Date input', () => {
        const testDate = 'I am not a date';
        const test = adapter.processDateInput(testDate);
        expect(test.toString()).toEqual('Invalid Date');
    });

    it('should display the date properly per the locale for Date input value of type Date', () => {
        const testDate = new Date('12/02/2001');
        const test = adapter.prepDateStringOutput(testDate);
        expect(test).toEqual('12/2/2001');
    });

    it('should display the date properly per the locale for Date input value of shortDate string', () => {
        const testDate = '12/02/2001';
        const test = adapter.prepDateStringOutput(testDate);
        expect(test).toEqual('12/2/2001');
    });

    it('should display the date properly per the locale for Date input value of type string', () => {
        const testDate = 'Dec 2 2001';
        const test = adapter.prepDateStringOutput(testDate);
        expect(test).toEqual('12/2/2001');
    });

    it('should create a new date from year, month, day', () => {
        const test = adapter.newDateFromArray(2017, 5, 15);
        expect(test).toEqual(new Date(2017, 5, 15));
    });

    it('should create a new date given no value', () => {
        const expectedDate = new Date();
        const test = adapter.newDateFromValue(undefined);
        expect(moment(test).isSame(expectedDate, 'day')).toBeTruthy();
    });

    it('should create a new date given a date value', () => {
        const test = adapter.newDateFromValue(new Date(2017, 5, 5));
        expect(test).toEqual(new Date(2017, 5, 5));
    });

    it('should return a new date from year, month', () => {
        const test = adapter.getMonthInfo(2017, 5);
        expect(test).toEqual(new Date(2017, 5, 15));
    });

    it('should return days in a month from year, month', () => {
        const test = adapter.daysInMonth(2018, 4);
        expect(test).toEqual(31);
    });

    it('should evaluate equality of calendar selection and comparator date', () => {
        const test = adapter.sameDay({year: 2018, month: 1, day: 15}, new Date(2018, 1, 15));
        expect(test).toBeTruthy();
    });

    it('should return a formatted day for header', () => {
        const test = adapter.prepHeaderDate(new Date(2018, 4, 15));
        expect(test).toEqual('Tue, May 15');
    });

    it('should return a formatted year for header', () => {
        const test = adapter.prepHeaderYear(new Date(2018, 4, 15));
        expect(test).toEqual('2018');
    });

    it('should return the month index of a given date', () => {
        const test = adapter.month(new Date(2018, 4, 15));
        expect(test).toEqual(4);
    });

    it('should return the year of a given date', () => {
        const test = adapter.year(new Date(2018, 4, 15));
        expect(test).toEqual(2018);
    });

    it('should return the day index of firstOfTheMonth of a given date', () => {
        const test = adapter.dayOfFirst(2018, 3);
        expect(test).toEqual(0);
    });

    it('should evaluate true if a given date is outside of the eligible range', () => {
        const test = adapter.isDisabled(
            new Date(2018, 4, 15),
            {minDate: new Date(2018, 4, 20), maxDate: new Date(2018, 4, 25)}
        );
        expect(test).toEqual(true);
    });

    it('should evaluate false if a given date is inside of the eligible range', () => {
        const test = adapter.isDisabled(
            new Date(2018, 4, 15),
            {minDate: new Date(2018, 4, 10), maxDate: new Date(2018, 4, 25)}
        );
        expect(test).toEqual(false);
    });

    it('should evaluate false if not given a range', () => {
        const test = adapter.isDisabled(
            new Date(2018, 4, 15), null);
        expect(test).toEqual(false);
    });

    it('should evaluate false if not given an empty range', () => {
        const test = adapter.isDisabled(
            new Date(2018, 4, 15), {});
        expect(test).toEqual(false);
    });

    it('should return the weekdays', () => {
        const result = adapter.weekdays();

        expect(result.length).toEqual(7);
        expect(result[0]).toBe('Sun');
        expect(result[6]).toBe('Sat');
    });

    it('should return the months', () => {
        const result = adapter.months();

        expect(result.length).toEqual(12);
        expect(result[0]).toBe('Jan');
        expect(result[11]).toBe('Dec');
    });
});

describe('service: NativeDateAdapter locale: en-GB', () => {
    let adapter: DateAdapter<any>;

    beforeEach(() => {
        adapter = new NativeDateAdapter('en-GB');
    });

    it('should process Date input of type Date properly', () => {
        const testDate = new Date('12/02/2001');
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(testDate);
    });

    it('should process Date input of shortDate string properly for locale, slash separation', () => {
        const testDate = '12/02/2001';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 1, 12));
    });

    it('should process Date input of shortDate string properly for locale, dash separation', () => {
        const testDate = '12-02-2001';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 1, 12));
    });

    it('should process Date input of shortDate string properly for locale, period separation', () => {
        const testDate = '12.02.2001';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 1, 12));
    });

    it('should process Date input of digit-only string properly for locale', () => {
        const testDate = '12022001';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 1, 12));
    });

    it('should process Date input of type string properly for locale', () => {
        const testDate = 'Dec 2 2001';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 11, 2));
    });

    it('should identify invalid Date input', () => {
        const testDate = 'I am not a date';
        const test = adapter.processDateInput(testDate);
        expect(test.toString()).toEqual('Invalid Date');
    });

    it('should display the date properly per the locale for Date input value of type Date', () => {
        const testDate = new Date('12/02/2001');
        const test = adapter.prepDateStringOutput(testDate);
        expect(test).toEqual('02/12/2001');
    });

    it('should display the date properly per the locale for Date input value of shortDate string', () => {
        const testDate = '12/02/2001';
        const test = adapter.prepDateStringOutput(testDate);
        expect(test).toEqual('12/02/2001');
    });

    it('should display the date properly per the locale for Date input value of type string', () => {
        const testDate = 'Dec 2 2001';
        const test = adapter.prepDateStringOutput(testDate);
        expect(test).toEqual('02/12/2001');
    });

    it('should display an empty string for falsy input', () => {
        const testDate = undefined;
        const test = adapter.prepDateStringOutput(testDate);
        expect(test).toEqual('');
    });

    it('should process ambigious short-string dates predictably', () => {
        let testDate = '1/5/2018';
        let test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 4, 1));

        testDate = '5/1/2018';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 5));

        testDate = '1/20/2018';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 20));

        testDate = '20/1/2018';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 20));

        testDate = '2018/1/5';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 4, 1));

        testDate = '2018/5/1';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 5));

        testDate = '2018/1/20';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 20));

        testDate = '2018/20/1';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 20));
    });

    it('should create a new date from year, month, day', () => {
        const test = adapter.newDateFromArray(2017, 5, 15);
        expect(test).toEqual(new Date(2017, 5, 15));
    });

    it('should create a new date given no value', () => {
        const expectedDate = new Date();
        const test = adapter.newDateFromValue(undefined);
        expect(moment(test).isSame(expectedDate, 'day')).toBeTruthy();
    });

    it('should create a new date given a date value', () => {
        const test = adapter.newDateFromValue(new Date(2017, 5, 5));
        expect(test).toEqual(new Date(2017, 5, 5));
    });

    it('should return a new date from year, month', () => {
        const test = adapter.getMonthInfo(2017, 5);
        expect(test).toEqual(new Date(2017, 5, 15));
    });

    it('should return days in a month from year, month', () => {
        const test = adapter.daysInMonth(2018, 4);
        expect(test).toEqual(31);
    });

    it('should evaluate equality of calendar selection and comparator date', () => {
        const test = adapter.sameDay({year: 2018, month: 1, day: 15}, new Date(2018, 1, 15));
        expect(test).toBeTruthy();
    });

    it('should return a formatted day for header', () => {
        const test = adapter.prepHeaderDate(new Date(2018, 4, 15));
        expect(test).toEqual('Tue, May 15');
    });

    it('should return a formatted year for header', () => {
        const test = adapter.prepHeaderYear(new Date(2018, 4, 15));
        expect(test).toEqual('2018');
    });

    it('should return the month index of a given date', () => {
        const test = adapter.month(new Date(2018, 4, 15));
        expect(test).toEqual(4);
    });

    it('should return the year of a given date', () => {
        const test = adapter.year(new Date(2018, 4, 15));
        expect(test).toEqual(2018);
    });

    it('should return the day index of firstOfTheMonth of a given date', () => {
        const test = adapter.dayOfFirst(2018, 3);
        expect(test).toEqual(0);
    });

    it('should evaluate true if a given date is outside of the eligible range', () => {
        const test = adapter.isDisabled(
            new Date(2018, 4, 15),
            {minDate: new Date(2018, 4, 20), maxDate: new Date(2018, 4, 25)}
        );
        expect(test).toEqual(true);
    });

    it('should evaluate false if a given date is inside of the eligible range', () => {
        const test = adapter.isDisabled(
            new Date(2018, 4, 15),
            {minDate: new Date(2018, 4, 10), maxDate: new Date(2018, 4, 25)}
        );
        expect(test).toEqual(false);
    });

    it('should return the weekdays', () => {
        const result = adapter.weekdays();

        expect(result.length).toEqual(7);
        expect(result[0]).toBe('Sun');
        expect(result[6]).toBe('Sat');
    });

    it('should return the months', () => {
        const result = adapter.months();

        expect(result.length).toEqual(12);
        expect(result[0]).toBe('Jan');
        expect(result[11]).toBe('Dec');
    });
});


describe('service: NativeDateAdapter locale: ja', () => {
    let adapter: DateAdapter<any>;

    beforeEach(() => {
        adapter = new NativeDateAdapter('ja');
    });

    it('should process Date input of type Date properly', () => {
        const testDate = new Date('12/02/2001');
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(testDate);
    });

    it('should process Date input of shortDate string properly for locale, slash separation', () => {
        const testDate = '2001/02/12';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 1, 12));
    });

    it('should process Date input of shortDate string properly for locale, dash separation', () => {
        const testDate = '2001-02-12';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 1, 12));
    });

    it('should process Date input of shortDate string properly for locale, period separation', () => {
        const testDate = '2001.02.12';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 1, 12));
    });

    it('should process Date input of digit-only string properly for locale', () => {
        const testDate = '20010212';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 1, 12));
    });

    it('should process Date input of shortDate string properly for locale, various character separation', () => {
        const testDate = '2001年2月12日';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 1, 12));
    });

    it('should process Date input of type string properly for locale', () => {
        const testDate = 'Dec 2 2001';
        const test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2001, 11, 2));
    });

    it('should process ambigious short-string dates predictably', () => {
        let testDate = '1/5/2018';
        let test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 5));

        testDate = '5/1/2018';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 4, 1));

        testDate = '1/20/2018';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 20));

        testDate = '20/1/2018';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 20));

        testDate = '2018/1/5';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 5));

        testDate = '2018/5/1';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 4, 1));

        testDate = '2018/1/20';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 20));

        testDate = '2018/20/1';
        test = adapter.processDateInput(testDate);
        expect(test).toEqual(new Date(2018, 0, 20));
    });

    it('should identify invalid Date input', () => {
        const testDate = 'I am not a date';
        const test = adapter.processDateInput(testDate);
        expect(test.toString()).toEqual('Invalid Date');
    });

    it('should display the date properly per the locale for Date input value of type Date', () => {
        const testDate = new Date('12/02/2001');
        const test = adapter.prepDateStringOutput(testDate);
        expect(test).toEqual('2001/12/2');
    });

    it('should display the date properly per the locale for Date', () => {
        const testDate = new Date(2018, 11, 15);
        const test = adapter.prepDateStringOutput(testDate);
        expect(test).toEqual('2018/12/15');
    });

    it('should create a new date from year, month, day', () => {
        const test = adapter.newDateFromArray(2017, 5, 15);
        expect(test).toEqual(new Date(2017, 5, 15));
    });

    it('should create a new date given no value', () => {
        const expectedDate = new Date();
        const test = adapter.newDateFromValue(undefined);
        expect(moment(test).isSame(expectedDate, 'day')).toBeTruthy();
    });

    it('should create a new date given a date value', () => {
        const test = adapter.newDateFromValue(new Date(2017, 5, 5));
        expect(test).toEqual(new Date(2017, 5, 5));
    });

    it('should return a new date from year, month', () => {
        const test = adapter.getMonthInfo(2017, 5);
        expect(test).toEqual(new Date(2017, 5, 15));
    });

    it('should return days in a month from year, month', () => {
        const test = adapter.daysInMonth(2018, 4);
        expect(test).toEqual(31);
    });

    it('should evaluate equality of calendar selection and comparator date', () => {
        const test = adapter.sameDay({year: 2018, month: 1, day: 15}, new Date(2018, 1, 15));
        expect(test).toBeTruthy();
    });

    it('should return a formatted day for header', () => {
        const test = adapter.prepHeaderDate(new Date(2018, 4, 15));
        expect(test).toEqual('火, 5月 15');
    });

    it('should return a formatted year for header', () => {
        const test = adapter.prepHeaderYear(new Date(2018, 4, 15));
        expect(test).toEqual('2018');
    });

    it('should return the month index of a given date', () => {
        const test = adapter.month(new Date(2018, 4, 15));
        expect(test).toEqual(4);
    });

    it('should return the year of a given date', () => {
        const test = adapter.year(new Date(2018, 4, 15));
        expect(test).toEqual(2018);
    });

    it('should return the day index of firstOfTheMonth of a given date', () => {
        const test = adapter.dayOfFirst(2018, 3);
        expect(test).toEqual(0);
    });

    it('should evaluate true if a given date is outside of the eligible range', () => {
        const test = adapter.isDisabled(
            new Date(2018, 4, 15),
            {minDate: new Date(2018, 4, 20), maxDate: new Date(2018, 4, 25)}
        );
        expect(test).toEqual(true);
    });

    it('should evaluate false if a given date is inside of the eligible range', () => {
        const test = adapter.isDisabled(
            new Date(2018, 4, 15),
            {minDate: new Date(2018, 4, 10), maxDate: new Date(2018, 4, 25)}
        );
        expect(test).toEqual(false);
    });

    it('should return the weekdays', () => {
        const result = adapter.weekdays();

        expect(result.length).toEqual(7);
        expect(result[0]).toBe('日');
        expect(result[6]).toBe('土');
    });

    it('should return the months', () => {
        const result = adapter.months();

        expect(result.length).toEqual(12);
        expect(result[0]).toBe('1月');
        expect(result[11]).toBe('12月');
    });
});
