import {CUSTOM_ELEMENTS_SCHEMA, LOCALE_ID} from '@angular/core';
import {TestBed, ComponentFixture, async} from '@angular/core/testing';
import {FormsModule, ReactiveFormsModule} from '@angular/forms';

import {DatePickerDirective} from '@anjuna/angular-core';
import {
    StubDatePickerNGMComponent,
    StubDatePickerMDComponent,
    StubDatePickerNoParentNoControlComponent
} from '@anjuna/angular-core/testing/src/date-picker.mock';
import {DateAdapter, NativeDateAdapter} from '@anjuna/angular-core/lib/forms/services/date-picker.service';

describe('Directive: date-picker - template driven', () => {
    let component: StubDatePickerNGMComponent;
    let comp;
    let fixture: ComponentFixture<StubDatePickerNGMComponent>;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            imports: [
                FormsModule,
            ],
            declarations: [
                DatePickerDirective,
                StubDatePickerNGMComponent
            ],
            schemas: [
                CUSTOM_ELEMENTS_SCHEMA
            ],
            providers: [
                {
                    provide: DateAdapter,
                    useClass: NativeDateAdapter,
                    deps: [LOCALE_ID]
                }
            ]
        }).compileComponents().then(() => {
            fixture = TestBed.createComponent(StubDatePickerNGMComponent);
            component = fixture.componentInstance;
            comp = component as any;
            fixture.detectChanges();
        });
    }));

    describe('Public API', () => {
        it('should exist', () => {
            expect(component.directive).toBeTruthy();
        });

        it('should have a value', () => {
            expect(component.directive.value).toBeTruthy();
        });

        it('should be able to set adapter', () => {
            component.directive.adapter = 'i am strategy';
            expect((component.directive.adapterStrategy as any)).toEqual('i am strategy');
        });

        it('should cancel the listener function if exists', () => {
            component.directive['listenerFn'] = () => {};
            spyOn((component.directive as any), 'listenerFn');

            component.directive.ngOnDestroy();

            expect(component.directive['listenerFn']).toHaveBeenCalled();
        });

        it('should respond to anjCalendarSelection', () => {
            comp.directive.innerDateValue = '1/1/2001';
            spyOn(comp.directive, 'onTouch');
            spyOn(comp.directive, 'updateInnerValue');
            spyOn(comp.directive.valueChange, 'emit');

            component.directive.onCalendarSelect({detail: '11/11/2001'});

            expect(comp.directive.onTouch).toHaveBeenCalled();
            expect(comp.directive.updateInnerValue).toHaveBeenCalled();
            expect(comp.directive.valueChange.emit).toHaveBeenCalled();
        });

        it('should respond to anjInvalid', () => {
            comp.directive.innerDateValue = '1/1/2001';
            comp.directive.invalid = true;
            spyOn(comp.directive, 'isInvalid');
            spyOn(comp.directive.changeRef, 'markForCheck');
            spyOn(comp.directive._element.nativeElement, 'dispatchEvent');

            component.directive.onAnjInvalid({});

            expect(comp.directive.isInvalid).toHaveBeenCalled();
            expect(comp.directive.changeRef.markForCheck).toHaveBeenCalled();
            expect(comp.directive._element.nativeElement.dispatchEvent).toHaveBeenCalled();
        });

        it('should not respond to anjInvalid if event detail equals current invalid status', () => {
            comp.directive.innerDateValue = '1/1/2001';
            comp.directive.invalid = true;
            spyOn(comp.directive.changeRef, 'markForCheck');
            spyOn(comp.directive._element.nativeElement, 'dispatchEvent');

            component.directive.onAnjInvalid({detail: true});

            expect(comp.directive.changeRef.markForCheck).not.toHaveBeenCalled();
            expect(comp.directive._element.nativeElement.dispatchEvent).not.toHaveBeenCalled();
        });

        it('should mark the control as touched and update inner value when onInput is called', () => {
            spyOn(component.directive, 'onTouch');
            spyOn((component.directive as any), 'updateInnerValue');
            component.directive.onInput({
                target: {
                    value: new Date(),
                    tagName: 'ANJ-INPUT'
                }
            });

            expect(component.directive.onTouch).toHaveBeenCalled();
            expect((component.directive as any).updateInnerValue).toHaveBeenCalled();
        });

        it('should do nothing on anjBlur if event is not from a anj-input', () => {
            spyOn(component.directive, 'onTouch');
            spyOn((component.directive as any), 'updateInnerValue');
            component.directive.onInput({
                target: {
                    value: 'abc',
                    tagName: 'ANJ-BUTTON'
                }
            });

            expect(component.directive.onTouch).not.toHaveBeenCalled();
            expect((component.directive as any).updateInnerValue).not.toHaveBeenCalled();
        });
    });

    describe('Private API', () => {
        it('should update the value and propogate that', () => {
            component.directive['innerDateValue'] = new Date(2012, 1, 1);
            spyOn((component.directive as any), 'propogateInnerValueChange').and.callThrough();

            const testDate = new Date(2018, 5, 5);

            component.directive['updateInnerValue'](testDate);

            expect((component.directive as any).propogateInnerValueChange).toHaveBeenCalled();
            expect((component.directive as any).innerDateValue).toBe(testDate);
        });

        it('should listen to given element', () => {
            component.directive['innerDateValue'] = new Date(2012, 1, 1);
            spyOn((component.directive as any), 'propogateInnerValueChange').and.callThrough();

            const testDate = new Date(2018, 5, 5);

            component.directive['updateInnerValue'](testDate);

            expect((component.directive as any).propogateInnerValueChange).toHaveBeenCalled();
            expect((component.directive as any).innerDateValue).toBe(testDate);
        });

        it('should not re-evaluate invalid status if no ngControl', () => {
            comp.directive.ngControl = undefined;
            comp.directive.invalid = undefined;

            comp.directive.isInvalid();

            expect(comp.directive.invalid).toBeUndefined();
        });

        it('should re-evaluate invalid status', () => {
            comp.directive.ngControl = {invalid: true};
            comp.directive.parent = {submitted: true};

            comp.directive.isInvalid();

            expect(comp.directive.invalid).toBeTruthy();
        });
    });
});

describe('Directive: date-picker - model driven', () => {
    let component: StubDatePickerMDComponent;
    let fixture: ComponentFixture<StubDatePickerMDComponent>;
    let dir: any;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            imports: [
                ReactiveFormsModule,
            ],
            declarations: [
                DatePickerDirective,
                StubDatePickerMDComponent
            ],
            schemas: [
                CUSTOM_ELEMENTS_SCHEMA
            ],
            providers: [
                {
                    provide: DateAdapter,
                    useClass: NativeDateAdapter,
                    deps: [LOCALE_ID]
                }
            ]
        }).compileComponents().then(() => {
            fixture = TestBed.createComponent(StubDatePickerMDComponent);
            component = fixture.componentInstance;
            dir = component.directive as any;
            fixture.detectChanges();
        });
    }));

    describe('Public API', () => {
        it('should exist', () => {
            expect(component.directive).toBeTruthy();
        });

        it('should take the adapter from the template', () => {
            expect(component.directive.adapterStrategy).toBeTruthy();
        });
    });

    describe('Private API', () => {
        it('should emit a value change when propogate is called', () => {
            spyOn(component.directive.valueChange, 'emit');
            spyOn(component.directive['ngControl'].control, 'setValue');

            component.directive['propogateInnerValueChange']();

            expect(component.directive.valueChange.emit).toHaveBeenCalled();
            expect(component.directive['ngControl'].control.setValue).toHaveBeenCalled();
        });
    });
});

describe('Directive: date-picker - model driven (no parent, no  control)', () => {
    let component: StubDatePickerNoParentNoControlComponent;
    let fixture: ComponentFixture<StubDatePickerNoParentNoControlComponent>;
    let dir: any;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            imports: [
                ReactiveFormsModule,
            ],
            declarations: [
                DatePickerDirective,
                StubDatePickerNoParentNoControlComponent
            ],
            schemas: [
                CUSTOM_ELEMENTS_SCHEMA
            ],
            providers: [
                {
                    provide: DateAdapter,
                    useClass: NativeDateAdapter
                },
                {
                    provide: LOCALE_ID,
                    useValue: undefined
                }
            ]
        }).compileComponents().then(() => {
            fixture = TestBed.createComponent(StubDatePickerNoParentNoControlComponent);
            component = fixture.componentInstance;
            dir = component.directive as any;
            fixture.detectChanges();
        });
    }));

    describe('Public API', () => {
        it('should exist', () => {
            expect(component.directive).toBeTruthy();
        });

        it('should have applicable functions for ControlValueAccessor', () => {
            component.directive.onChange();
            component.directive.onTouch();
        });
    });

    describe('Private API', () => {
        it('should emit a value change when propogate is called, but not the ngControl if missing', () => {
            spyOn(component.directive.valueChange, 'emit');
            (component as any).directive.ngControl = undefined;

            component.directive['propogateInnerValueChange']();

            expect(component.directive.valueChange.emit).toHaveBeenCalled();
        });
    });
});
