import {async, ComponentFixture, TestBed, fakeAsync, tick} from '@angular/core/testing';
import {DebugElement} from '@angular/core';
import {By} from '@angular/platform-browser';

import {StubTransitionComponent} from '../testing/src/transition.mock';
import {TransitionState, TransitionDirective} from '@anjuna/angular-core';

const stubEvent = (): any => {
    return {
        type: 'transitionend',
        stopPropagation: () => {}
    };
};

describe('directive: anjTransition', () => {
    let component: StubTransitionComponent;
    let fixture: ComponentFixture<StubTransitionComponent>;

    beforeEach(async(() => {
        TestBed.configureTestingModule({
            declarations: [
                StubTransitionComponent,
                TransitionDirective
            ]
        });
    }));

    beforeEach(() => {
        fixture = TestBed.createComponent(StubTransitionComponent);
        component = fixture.componentInstance;

        fixture.detectChanges();
    });

    it('should exist', () => {
        expect(typeof component.directives.first).not.toBe('undefined');
    });

    it('should have a default name of "transition"', () => {
        expect(component.directives.first.name).toBe('transition');
    });

    it('should bind to the name when provided', () => {
        expect(component.directives.last.name).toBe('test');
    });

    it('should have a default mode of "in-out"', () => {
        expect(component.directives.first.mode).toBe('in-out');
    });

    it('should bind to the mode when provided', () => {
        expect(component.directives.last.mode).toBe('in');
    });

    it('should have a default state (anjTransition) of undefined', () => {
        expect(component.directives.first.anjTransition).toBe(undefined);
    });

    it('should have an event emitter for onEnd', () => {
        expect(component.directives.first.end).not.toBe(undefined);
    });

    it('should add the enter to class to itself when initEnter is called', () => {
        const dir = component.directives.last as any;

        dir.initEnter();
        const block: DebugElement = fixture.debugElement.query(By.css('.block2'));
        expect(block.nativeElement.classList).toContain('anj-test-enter-to');
    });

    it('should usually trigger the enter transition when transitionIn is called', () => {
        const dir = component.directives.last as any;
        spyOn(dir, 'endTransition');

        dir.initEnter();
        dir.transitionIn();

        const block: DebugElement = fixture.debugElement.query(By.css('.block2'));
        expect(block.nativeElement.classList).not.toContain('anj-test-enter-to');
        expect(block.nativeElement.classList).not.toContain('anj-test-leave');
        expect(block.nativeElement.classList).not.toContain('anj-test-leave-active');
        expect(block.nativeElement.classList).toContain('anj-test-enter');
        expect(block.nativeElement.classList).toContain('anj-test-enter-active');

        expect(dir.endTransition).not.toHaveBeenCalled();
    });

    it('should skip the enter animation when the mode is "out"', () => {
        const dir = component.directives.last as any;
        spyOn(dir, 'endTransition');

        dir.mode = 'out';
        dir.transitionIn();

        const block: DebugElement = fixture.debugElement.query(By.css('.block2'));
        expect(block.nativeElement.classList).not.toContain('anj-test-enter-to');
        expect(dir.endTransition).toHaveBeenCalled();
    });

    it('should skip the leave animation when the mode is "in"', fakeAsync(() => {
        const dir = component.directives.last as any;
        spyOn(dir, 'transitionIn');

        dir.transitionIn();
        dir.transitionOut();

        const block: DebugElement = fixture.debugElement.query(By.css('.block2'));
        expect(block.nativeElement.classList).not.toContain('anj-test-leave-active');
        expect(block.nativeElement.classList).not.toContain('anj-test-leave');
        expect(block.nativeElement.classList).toContain('anj-test-enter-to');

        tick();

        expect(dir.transitionIn).toHaveBeenCalled();
    }));

    it('should usually trigger the leave animation when transitionOut is called', fakeAsync(() => {
        const dir = component.directives.first as any;
        spyOn(dir, 'transitionIn');

        dir.transitionOut();

        const block: DebugElement = fixture.debugElement.query(By.css('.block1'));
        expect(block.nativeElement.classList).toContain('anj-transition-leave-active');
        expect(block.nativeElement.classList).toContain('anj-transition-leave');

        tick();

        expect(dir.transitionIn).not.toHaveBeenCalled();
    }));

    it('should call delayed transitionIn', fakeAsync(() => {
        const dir = component.directives.last as any;
        dir.anjTransition = true;
        dir.mode = 'in';
        spyOn(dir, 'transitionIn');

        dir.transitionOut();

        const block: DebugElement = fixture.debugElement.query(By.css('.block2'));
        expect(block.nativeElement.classList).not.toContain('anj-test-leave-active');
        expect(block.nativeElement.classList).not.toContain('anj-test-leave');
        expect(block.nativeElement.classList).toContain('anj-test-enter-to');

        tick();

        expect(dir.transitionIn).toHaveBeenCalled();
    }));

    it('should clear all transition classes on transition end', () => {
        const dir = component.directives.first as any;

        dir.transitionIn();
        dir.endTransition();

        const block: DebugElement = fixture.debugElement.query(By.css('.block1'));
        expect(block.nativeElement.classList).not.toContain('anj-transition-leave-active');
        expect(block.nativeElement.classList).not.toContain('anj-transition-leave');
        expect(block.nativeElement.classList).not.toContain('anj-transition-enter');
        expect(block.nativeElement.classList).not.toContain('anj-transition-enter-active');
    });

    it('should add the enter-to class if the state is falsey on transitionEnd', () => {
        const dir = component.directives.first as any;
        dir.transitionIn();
        dir.endTransition();

        const block: DebugElement = fixture.debugElement.query(By.css('.block1'));
        expect(block.nativeElement.classList).toContain('anj-transition-enter-to');
    });

    it('should not add the entro-to class if the state is truthy on transitionEnd', () => {
        const dir = component.directives.first as any;
        dir.anjTransition = 'foo';
        dir.transitionIn();
        dir.endTransition();

        const block: DebugElement = fixture.debugElement.query(By.css('.block1'));
        expect(block.nativeElement.classList).not.toContain('anj-transition-enter-to');
    });

    it('should call initEnter when there is no state value onChanges', () => {
        const dir = component.directives.first as any;

        spyOn(dir, 'initEnter');

        const changes: any = {
            anjTransition: {
                previousValue: undefined,
                currentValue: undefined,
            }
        };

        dir.ngOnChanges(changes);

        expect(dir.initEnter).toHaveBeenCalled();
    });

    it('should call transtionIn when the the state goes from falsey to truthy', () => {
        const dir = component.directives.first as any;

        spyOn(dir, 'transitionIn');

        const changes: any = {
            anjTransition: {
                previousValue: undefined,
                currentValue: 'foo',
            }
        };

        dir.ngOnChanges(changes);

        expect(dir.transitionIn).toHaveBeenCalled();
    });

    it ('should default to calling transition out', () => {
        const dir = component.directives.first as any;

        spyOn(dir, 'transitionOut');

        const changes: any = {
            anjTransition: {
                previousValue: 'bar',
                currentValue: undefined,
            }
        };
        dir.ngOnChanges(changes);

        expect(dir.transitionOut).toHaveBeenCalled();
    });

    it ('should ignore transition events not in our supported ones', () => {
        const dir = component.directives.first as any;
        const event = new Event('transitionend') as any;

        event.propertyName = 'color';
        spyOn(event, 'stopPropagation');

        dir.handleTransition(event);

        expect(event.stopPropagation).not.toHaveBeenCalled();
    });

    it('should stop transition events from bubbling up', () => {
        const dir = component.directives.first as any;
        const event = stubEvent();

        event.propertyName = 'opacity';
        event.target = component.el1.nativeElement;
        spyOn(event, 'stopPropagation');

        dir.handleTransition(event);

        expect(event.stopPropagation).toHaveBeenCalled();
    });

    it('should emit the on end event when the transition ends', () => {
        const dir = component.directives.first as any;
        const event = stubEvent();

        spyOn(dir.end, 'emit');
        event.propertyName = 'opacity';
        event.target = component.el1.nativeElement;

        dir.transitionIn();
        dir.handleTransition(event);

        expect(dir.end.emit).toHaveBeenCalledWith(TransitionState.Enter);
    });

    it('should end the transition after transition enter', () => {
        const dir = component.directives.first as any;
        const event = stubEvent();

        spyOn(dir, 'endTransition');
        event.propertyName = 'opacity';
        event.target = component.el1.nativeElement;

        dir.transitionIn();
        dir.handleTransition(event);

        expect(dir.endTransition).toHaveBeenCalled();
    });

    it('should end the transition after transition leave if there is no state', () => {
        const dir = component.directives.first as any;
        const event = stubEvent();

        spyOn(dir, 'endTransition');
        event.propertyName = 'opacity';
        event.target = component.el1.nativeElement;

        dir.transitionOut();
        dir.handleTransition(event);

        expect(dir.endTransition).toHaveBeenCalled();
    });

    it('should start the enter transition after leaving if there is state', () => {
        const dir = component.directives.first as any;
        const event = stubEvent();

        spyOn(dir, 'endTransition');
        spyOn(dir, 'transitionIn');
        event.propertyName = 'opacity';
        event.target = component.el1.nativeElement;

        dir.anjTransition = 'foo';
        dir.transitionOut();
        dir.handleTransition(event);

        expect(dir.endTransition).not.toHaveBeenCalled();
        expect(dir.transitionIn).toHaveBeenCalled();
    });
});

